
//display
#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define SCREEN_WIDTH 128 // OLED Breite in Pixel
#define SCREEN_HEIGHT 32 // OLED Höhe in Pixel
#define OLED_RESET -1
#define SCREEN_ADDRESS 0x3C
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

//Kodierschalter mit Taster
#define TASTER 4
#define encoder0PinA  3
#define encoder0PinB  2

//Variablen
volatile unsigned int encoder0Pos = 0;
int Effekt;
int Effektvorwahl;
int Farbe;

String textEffekt[20] = {
  "Aus",      // 0
  "Bunt",     // 1
  "Feuer",    // 2
  "Funkeln",  // 3
  "Funkeln 2",// 4
  "LGBTQ",    // 5
  "Rot",      // 6
  "Gruen",    // 7
  "Blau",     // 8
  "Gelb",     // 9
  "Violett",  // 10
  "Tuerkis",   // 11
  "Laufl. RT", // 12
  "Laufl. WS", // 13
  "Laufl. BL", // 14
  "Theater",   // 15
  "Strobo",    // 16
  "KITT",      // 17
  "Ukraine",   // 18
  "OTTO"};     // 19

const int SHORT_PRESS_TIME = 1000; // 1000 milliseconds
const int LONG_PRESS_TIME  = 1000;

int           lastState = LOW;  // the previous state from the input pin
int           currentState = HIGH;     // the current reading from the input pin
unsigned long pressedTime  = 0;
unsigned long releasedTime = 0;
long          pressDuration = 0;

byte menuCount = 0;
byte dir = 0;
bool runState = false;


//LEDs
#include "FastLED.h"
#define PIN 5
#define NUM_LEDS 50
CRGB leds[NUM_LEDS];


void setup() {
  pinMode(encoder0PinA, INPUT);
  digitalWrite(encoder0PinA, HIGH); // internen Pull-Up aktivieren
  pinMode(encoder0PinB, INPUT);
  digitalWrite(encoder0PinB, HIGH); // internen Pull-Up aktivieren
  pinMode(TASTER, INPUT_PULLUP);
  pinMode(LED_BUILTIN, OUTPUT);

  FastLED.addLeds<WS2811, PIN, GRB>(leds, NUM_LEDS).setCorrection( TypicalLEDStrip );

  if (!display.begin(SSD1306_SWITCHCAPVCC, SCREEN_ADDRESS)) {
    Serial.println(F("OLED Display nicht erkannt"));
    for (;;); // Sackgasse, wenn OLED nicht funzt
  }

  //SplashScreen von Adafruit unterdrücken
  display.clearDisplay();
  display.display();


  //Willkommenstext anzeigen
  display.setTextSize(3);
  display.setTextColor(WHITE);
  display.setCursor(30, 0);
  display.println("Rock");
  display.display();
  delay(500);
  display.clearDisplay();
  display.display();
  delay(50);
  display.setCursor(30, 0);
  display.println("-O-");
  display.display();
  delay(500);
  display.clearDisplay();
  display.display();
  delay(50);
  display.setCursor(30, 0);
  display.println("Mat");
  display.display();
  delay(500);
  display.clearDisplay();
  display.display();
  delay(100);
  display.setCursor(30, 0);
  display.println("2000");
  display.display();
  delay(500);
  display.clearDisplay();
  display.display();
  delay(100);
  // Clear the buffer
  display.clearDisplay();


  attachInterrupt(0, doEncoder, CHANGE);  // Pin vom Kodierschalter als interrupt 0 (Pin 2)
}

void loop() {


  CheckButtonTiming();
  Menu();
  display.clearDisplay();

  switch (Effekt) {
    case 1: //bunt
      Bunt(20, 20, false);
      break;
    case 2: //Feuer
      Feuer(55, 120, 2);
      break;
    case 3:
      Funkeln(random(255), random(255), random(255), 20);
      break;
    case 4:
      Funkeln2(0x05, 0x05, 0x05, 20, 200);
      break;
    case 5:
      LGBTQ(20);
      break;
    case 6: //Rot
      for ( int i = 0; i < NUM_LEDS; ++i) {
        setPixel(i, 255, 0, 0);
      }
      break;
    case 7: //Grün
      for ( int i = 0; i < NUM_LEDS; ++i) {
        setPixel(i, 0, 255, 0);
      }
      break;
    case 8: //Blau
      for ( int i = 0; i < NUM_LEDS; ++i) {
        setPixel(i, 0, 0, 255);
      }
      break;
    case 9: //Gelb
      for ( int i = 0; i < NUM_LEDS; ++i) {
        setPixel(i, 255, 255, 0);
      }
      break;
    case 10: //Violett
      for ( int i = 0; i < NUM_LEDS; ++i) {
        setPixel(i, 255, 0, 255);
      }
      break;
    case 11: //Türkis
      for ( int i = 0; i < NUM_LEDS; ++i) {
        setPixel(i, 0, 255, 255);
      }
      break;
    case 12: //Lauflicht Rot
      RunningLights(0xff, 0, 0, 50);
      break;
    case 13: //Lauflicht Weiß
      RunningLights(0xff, 0xff, 0xff, 50);
      break;
    case 14: //Lauflicht Blau
      RunningLights(0, 0, 0xff, 50);
      break;
    case 15: //Theater
      theaterChase(0xff,0,0,50);
      break;
    case 16: //Strobo
      Strobe(0xff, 0xff, 0xff, 10, 50, 1000);
      break;
          case 17: //KITT
      CylonBounce(0xff, 0, 0, 4, 10, 50);
      break;
    case 18: //Ukraine
      for ( int i = 0; i < 25; ++i) {
        setPixel(i, 255, 255, 0);
      }
      for ( int i = 26; i < NUM_LEDS; ++i) {
        setPixel(i, 0,0,255);
      }
      break;
    case 19: //OTTO
      for ( int i = 0; i < 10; ++i) {
        setPixel(i, 255, 255, 0);
      }
      for ( int i = 11; i < 20; ++i) {
        setPixel(i, 255,0,0);
      }
            for ( int i = 21; i < 30; ++i) {
        setPixel(i, 255,255,0);
      }
            for ( int i = 31; i < 40; ++i) {
        setPixel(i, 255,0,0);
      }
            for ( int i = 41; i < 50; ++i) {
        setPixel(i, 255,255,0);
      }
      break;
      
    default:
      for ( int i = 0; i < NUM_LEDS; ++i) {
        setPixel(i, 0, 0, 0);
      }
      break;
  }
  //delay(50);

  showStrip();
}


void CheckButtonTiming() {
  // read the state of the switch/button:
  currentState = digitalRead(TASTER);

  if (lastState == HIGH && currentState == LOW) {      // button is pressed
    pressedTime = millis();
  } else {
    if (lastState == LOW && currentState == HIGH) { // button is released
      releasedTime = millis();
      pressDuration = releasedTime - pressedTime;
      EvaluateButtonTiming();
    }
  }
  // save the the last state
  lastState = currentState;
}

void EvaluateButtonTiming() {
  if ( pressDuration < SHORT_PRESS_TIME ) {
    //Serial.println("Kurzer Tastendruck erkannt");
    //digitalWrite(pin_LED1, HIGH);
    digitalWrite(LED_BUILTIN, HIGH);
    //menuCount++;
    Effekt = Effektvorwahl ;

  } else {
    //digitalWrite(pin_LED1, LOW);
  }

  if ( pressDuration > LONG_PRESS_TIME ) {
    //Serial.println("Langer Tastendruck erkannt");
    Effekt = 0;
    Effektvorwahl = 0; // WIEDER AUF 0 SETZEN!
    digitalWrite(LED_BUILTIN, LOW);
  } else {
    //digitalWrite(pin_LED2, LOW);
  }

}

void Menu() {
  display.setTextSize(1);
  display.setTextColor(WHITE);

  display.setCursor(10, 0);
  display.println("Rock-O-Mat 2000");
  //---------------------------------
  display.setTextSize(2);
  display.setCursor(0, 10);
  //display.println("FX:");
  //display.setCursor(30, 10);

  if (Effekt == Effektvorwahl) {
    display.setTextColor(WHITE);
  }
  else {
    display.setTextColor(BLACK, WHITE);
  }
  display.println(textEffekt[Effektvorwahl]);
  display.display();
}


void doEncoder() {



  if (digitalRead(encoder0PinA) == HIGH) {
    delay(5);
    if (digitalRead(encoder0PinB) == LOW && encoder0Pos > 0) {
      encoder0Pos = encoder0Pos - 1;
      dir = 0;
      Effektvorwahl = encoder0Pos;
    }
    else {
      //encoder0Pos = encoder0Pos + 1;
      //dir = 1;

    }
  }
  else
  {
    if (digitalRead(encoder0PinB) == LOW ) {
      delay(5);
      encoder0Pos = encoder0Pos + 1;
      dir = 1;
      Effektvorwahl = encoder0Pos;
    }
    else {
      if (encoder0Pos > 0) {
        //encoder0Pos = encoder0Pos - 1;
        //dir = 0;

      }
    }
  }
}





void setPixelHeatColor (int Pixel, byte temperature) {
  // Scale 'heat' down from 0-255 to 0-191
  byte t192 = round((temperature / 255.0) * 191);

  // calculate ramp up from
  byte heatramp = t192 & 0x3F; // 0..63
  heatramp <<= 2; // scale up to 0..252

  // figure out which third of the spectrum we're in:
  if ( t192 > 0x80) {                    // hottest
    setPixel(Pixel, 255, 255, heatramp);
  } else if ( t192 > 0x40 ) {            // middle
    setPixel(Pixel, 255, heatramp, 0);
  } else {                               // coolest
    setPixel(Pixel, heatramp, 0, 0);
  }
}

void showStrip() {
#ifdef ADAFRUIT_NEOPIXEL_H
  // NeoPixel
  strip.show();
#endif
#ifndef ADAFRUIT_NEOPIXEL_H
  // FastLED
  FastLED.show();
#endif
}

void setPixel(int Pixel, byte red, byte green, byte blue) {
#ifdef ADAFRUIT_NEOPIXEL_H
  // NeoPixel
  strip.setPixelColor(Pixel, strip.Color(red, green, blue));
#endif
#ifndef ADAFRUIT_NEOPIXEL_H
  // FastLED
  leds[Pixel].r = red;
  leds[Pixel].g = green;
  leds[Pixel].b = blue;
#endif
}

void setAll(byte red, byte green, byte blue) {
  for (int i = 0; i < NUM_LEDS; i++ ) {
    setPixel(i, red, green, blue);
  }
  showStrip();
}

/////////////////
//// Effekte ////
/////////////////

void Feuer(int Cooling, int Sparking, int SpeedDelay) {
  static byte heat[NUM_LEDS];
  int cooldown;

  // Step 1.  Cool down every cell a little
  for ( int i = 0; i < NUM_LEDS; i++) {
    cooldown = random(0, ((Cooling * 10) / NUM_LEDS) + 2);

    if (cooldown > heat[i]) {
      heat[i] = 0;
    } else {
      heat[i] = heat[i] - cooldown;
    }
  }

  // Step 2.  Heat from each cell drifts 'up' and diffuses a little
  for ( int k = NUM_LEDS - 1; k >= 2; k--) {
    heat[k] = (heat[k - 1] + heat[k - 2] + heat[k - 2]) / 3;
  }

  // Step 3.  Randomly ignite new 'sparks' near the bottom
  if ( random(255) < Sparking ) {
    int y = random(7);
    heat[y] = heat[y] + random(160, 255);
    //heat[y] = random(160,255);
  }

  // Step 4.  Convert heat to LED colors
  for ( int j = 0; j < NUM_LEDS; j++) {
    setPixelHeatColor(j, heat[j] );
  }

  showStrip();
  delay(SpeedDelay);
}

void Bunt(int Anzahl, int SpeedDelay, boolean NurEine) {
  //setAll(0,0,0);

  for (int i = 0; i < Anzahl; i++) {
    setPixel(random(NUM_LEDS), random(0, 255), random(0, 255), random(0, 255));
    showStrip();
    delay(SpeedDelay);
    if (NurEine) {
      setAll(0, 0, 0);
    }
  }

  delay(SpeedDelay);
}

void Funkeln(byte red, byte green, byte blue, int SpeedDelay) {
  int Pixel = random(NUM_LEDS);
  setPixel(Pixel, red, green, blue);
  showStrip();
  delay(SpeedDelay);
  setPixel(Pixel, 0, 0, 0);
}

void Funkeln2(byte red, byte green, byte blue, int SparkleDelay, int SpeedDelay) {
  setAll(red, green, blue);

  int Pixel = random(NUM_LEDS);
  setPixel(Pixel, 0xff, 0xff, 0xff);
  showStrip();
  delay(SparkleDelay);
  setPixel(Pixel, red, green, blue);
  showStrip();
  delay(SpeedDelay);
}

void LGBTQ(int SpeedDelay) {
  byte *c;
  uint16_t i, j;

  for (j = 0; j < 256 * 5; j++) { // 5 cycles of all colors on wheel
    for (i = 0; i < NUM_LEDS; i++) {
      c = Wheel(((i * 256 / NUM_LEDS) + j) & 255);
      setPixel(i, *c, *(c + 1), *(c + 2));
    }
    showStrip();
    delay(SpeedDelay);
  }
}

byte * Wheel(byte WheelPos) {
  static byte c[3];

  if (WheelPos < 85) {
    c[0] = WheelPos * 3;
    c[1] = 255 - WheelPos * 3;
    c[2] = 0;
  } else if (WheelPos < 170) {
    WheelPos -= 85;
    c[0] = 255 - WheelPos * 3;
    c[1] = 0;
    c[2] = WheelPos * 3;
  } else {
    WheelPos -= 170;
    c[0] = 0;
    c[1] = WheelPos * 3;
    c[2] = 255 - WheelPos * 3;
  }

  return c;
}

void RunningLights(byte red, byte green, byte blue, int WaveDelay) {
  int Position = 0;

  for (int j = 0; j < NUM_LEDS * 2; j++)
  {
    Position++; // = 0; //Position + Rate;
    for (int i = 0; i < NUM_LEDS; i++) {
      // sine wave, 3 offset waves make a rainbow!
      //float level = sin(i+Position) * 127 + 128;
      //setPixel(i,level,0,0);
      //float level = sin(i+Position) * 127 + 128;
      setPixel(i, ((sin(i + Position) * 127 + 128) / 255)*red,
               ((sin(i + Position) * 127 + 128) / 255)*green,
               ((sin(i + Position) * 127 + 128) / 255)*blue);
    }

    showStrip();
    delay(WaveDelay);
  }
}
void theaterChase(byte red, byte green, byte blue, int SpeedDelay) {
  for (int j=0; j<10; j++) {  //do 10 cycles of chasing
    for (int q=0; q < 3; q++) {
      for (int i=0; i < NUM_LEDS; i=i+3) {
        setPixel(i+q, red, green, blue);    //turn every third pixel on
      }
      showStrip();
     
      delay(SpeedDelay);
     
      for (int i=0; i < NUM_LEDS; i=i+3) {
        setPixel(i+q, 0,0,0);        //turn every third pixel off
      }
    }
  }
}

void Strobe(byte red, byte green, byte blue, int StrobeCount, int FlashDelay, int EndPause){
  for(int j = 0; j < StrobeCount; j++) {
    setAll(red,green,blue);
    showStrip();
    delay(FlashDelay);
    setAll(0,0,0);
    showStrip();
    delay(FlashDelay);
  }
 
 delay(EndPause);
}

void CylonBounce(byte red, byte green, byte blue, int EyeSize, int SpeedDelay, int ReturnDelay){

  for(int i = 0; i < NUM_LEDS-EyeSize-2; i++) {
    setAll(0,0,0);
    setPixel(i, red/10, green/10, blue/10);
    for(int j = 1; j <= EyeSize; j++) {
      setPixel(i+j, red, green, blue);
    }
    setPixel(i+EyeSize+1, red/10, green/10, blue/10);
    showStrip();
    delay(SpeedDelay);
  }

  delay(ReturnDelay);

  for(int i = NUM_LEDS-EyeSize-2; i > 0; i--) {
    setAll(0,0,0);
    setPixel(i, red/10, green/10, blue/10);
    for(int j = 1; j <= EyeSize; j++) {
      setPixel(i+j, red, green, blue);
    }
    setPixel(i+EyeSize+1, red/10, green/10, blue/10);
    showStrip();
    delay(SpeedDelay);
  }
 
  delay(ReturnDelay);
}
